"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.anchor = exports.getGitlabId = exports.getGhostId = exports.getBasicGhostId = exports.getBitbucketId = exports.getGithubId = exports.getBasicGithubId = exports.getNodejsId = void 0;
const emoji_regex_1 = __importDefault(require("emoji-regex"));
// https://github.com/joyent/node/blob/192192a09e2d2e0d6bdd0934f602d2dbbf10ed06/tools/doc/html.js#L172-L183
exports.getNodejsId = (text, repetition) => {
    text = text.replace(/[^a-z0-9]+/g, '_');
    text = text.replace(/^_+|_+$/, '');
    text = text.replace(/^([^a-z])/, '_$1');
    // If no repetition, or if the repetition is 0 then ignore. Otherwise append '_' and the number.
    // An example may be found here: http://nodejs.org/api/domain.html#domain_example_1
    if (repetition) {
        text += '_' + repetition;
    }
    return text;
};
exports.getBasicGithubId = (text) => {
    return text
        // numeric character references
        .replace(/&#(\d+);/ig, (match, $1) => String.fromCharCode($1).toLowerCase())
        // to hyphen
        .replace(/ /g, '-')
        // escape codes
        .replace(/%([abcdef]|\d){2}/ig, '')
        // single chars that are removed
        .replace(/[/?!:[\]`.,()*"';{}+=<>~$|#@%^&¥–—]/g, '')
        // CJK punctuations that are removed
        .replace(/[。？！，、；：“”【】（）〔〕［］﹃﹄ ‘’﹁﹂—…－～《》〈〉「」]/g, '');
};
exports.getGithubId = (text, repetition) => {
    text = exports.getBasicGithubId(text);
    // If no repetition, or if the repetition is 0 then ignore. Otherwise append '-' and the number.
    if (repetition) {
        text += '-' + repetition;
    }
    // Strip emojis
    text = text.replace(emoji_regex_1.default(), '');
    return text;
};
exports.getBitbucketId = (text, repetition) => {
    text = 'markdown-header-' + exports.getBasicGithubId(text);
    // BitBucket condenses consecutive hyphens (GitHub doesn't)
    text = text.replace(/--+/g, '-');
    // If no repetition, or if the repetition is 0 then ignore. Otherwise append '_' and the number.
    // https://groups.google.com/d/msg/bitbucket-users/XnEWbbzs5wU/Fat0UdIecZkJ
    if (repetition) {
        text += '_' + repetition;
    }
    return text;
};
exports.getBasicGhostId = (text) => {
    return text.replace(/ /g, '')
        // escape codes are not removed
        // single chars that are removed
        .replace(/[/?:[\]`.,()*"';{}\-+=<>!@#%^&\\|]/g, '')
        // $ replaced with d
        .replace(/\$/g, 'd')
        // ~ replaced with t
        .replace(/~/g, 't');
};
exports.getGhostId = (text) => {
    text = exports.getBasicGhostId(text);
    // Repetitions not supported
    return text;
};
// see: https://github.com/gitlabhq/gitlabhq/blob/master/doc/user/markdown.md#header-ids-and-links
exports.getGitlabId = (text, repetition) => {
    text = text
        .replace(/<(.*)>(.*)<\/\1>/g, '$2') // html tags
        .replace(/!\[.*]\(.*\)/g, '') // image tags
        .replace(/\[(.*)]\(.*\)/, '$1') // url
        .replace(/\s+/g, '-') // All spaces are converted to hyphens
        .replace(/[/?!:[\]`.,()*"';{}+=<>~$|#@]/g, '') // All non-word text (e.g., punctuation, HTML) is removed
        .replace(/[。？！，、；：“”【】（）〔〕［］﹃﹄ ‘’﹁﹂—…－～《》〈〉「」]/g, '') // remove CJK punctuations
        .replace(/[-]+/g, '-') // duplicated hyphen
        .replace(/^-/, '') // ltrim hyphen
        .replace(/-$/, ''); // rtrim hyphen
    // If no repetition, or if the repetition is 0 then ignore. Otherwise append '-' and the number.
    if (repetition) {
        text += '-' + repetition;
    }
    return text;
};
exports.anchor = (header, mode, repetition, moduleName) => {
    mode = mode || 'github.com';
    let replace;
    let customEncodeURI = encodeURI;
    switch (mode) {
        case 'github.com':
            replace = exports.getGithubId;
            customEncodeURI = (uri) => {
                const newURI = encodeURI(uri);
                // encodeURI replaces the zero width joiner character
                // (used to generate emoji sequences, e.g.Female Construction Worker 👷🏼‍♀️)
                // github doesn't URL encode them, so we replace them after url encoding to preserve the zwj character.
                return newURI.replace(/%E2%80%8D/g, '\u200D');
            };
            break;
        case 'bitbucket.org':
            replace = exports.getBitbucketId;
            break;
        case 'gitlab.com':
            replace = exports.getGitlabId;
            break;
        case 'nodejs.org':
            if (!moduleName) {
                throw new Error('Need module name to generate proper anchor for ' + mode);
            }
            replace = (hd, repetition) => {
                return exports.getNodejsId(moduleName + '.' + hd, repetition);
            };
            break;
        case 'ghost.org':
            replace = exports.getGhostId;
            break;
        default:
            throw new Error('Unknown mode: ' + mode);
    }
    const asciiOnlyToLowerCase = (input) => {
        let result = '';
        [...Array(input.length).keys()].forEach(index => {
            if (input[index] >= 'A' && input[index] <= 'Z') {
                result += input[index].toLowerCase();
            }
            else {
                result += input[index];
            }
        });
        return result;
    };
    return '[' + header + '](#' + customEncodeURI(replace(asciiOnlyToLowerCase(header.trim()), repetition)) + ')';
};
exports.default = exports.anchor;
